@@ -1,22 +1,43 @@
 import requests
 import json
 from typing import Optional
+from .exceptions import *
 
 token = ""
 
 
-def initiate_payment(tx_ref: str, amount: float, currency: Optional[str], redirect_url: str, customer_name: str,
-                     payment_options: Optional[str], title: Optional[str], customer_email: str,
-                     customer_phone_number: Optional[str], description: Optional[str]) -> str:
-    """This is used to initiate standard payments. It takes in the arguments and returns the url to redirect users for
-    payments """
+def initiate_payment(tx_ref: str, amount: float, redirect_url: str, customer_email: str,
+                     customer_name: Optional[str] = None, currency: Optional[str] = None,
+                     customer_phone_number: Optional[str] = None, payment_options: Optional[str] = None,
+                     title: Optional[str] = None, description: Optional[str] = None) -> str:
+    """
+    This is used to initiate standard payments. It takes in the arguments and returns the url to redirect users for
+    payments
+    :param tx_ref: str
+    :param amount: float
+    :param redirect_url: str
+    :param customer_email: str
+    :param customer_name: Optional[str]
+    :param customer_phone_number: Optional[str]
+    :param currency: Optional[str]
+    :param payment_options: Optional[str]
+    :param title: Optional[str]
+    :param description: Optional[str]
+    :return: str
+    """
     payment_url = "https://api.flutterwave.com/v3/payments"
+
+    if token == "" or token is None:
+        raise TokenException(token=token, message="Authentication token absent")
+    if customer_email is None or customer_email == "":
+        raise CustomerDetailException(message="Customer email not provided")
+
     payload = json.dumps({
         "tx_ref": f"{tx_ref}",
         "amount": f"{amount}",
         "currency": f"{currency}".upper(),
         "redirect_url": f"{redirect_url}",
-        "payment_options": f"{payment_options}",
+        "payment_options": payment_options,
         "customer": {
             "email": f"{customer_email}",
             "phonenumber": f"{customer_phone_number}",
@@ -33,6 +54,10 @@ def initiate_payment(tx_ref: str, amount: float, currency: Optional[str], redire
     }
 
     response = requests.request(method="POST", url=payment_url, headers=headers, data=payload)
+    if response.status_code == 401:
+        raise TokenException(token=token, message='Invalid token provided')
+    if response.status_code == 400:
+        raise Exception(f"{response.json()['message']}")
     link = response.json()["data"]["link"]
     return link
 
@@ -44,24 +69,34 @@ def get_payment_details(trans_id: str) -> dict:
     """
     url = f"https://api.flutterwave.com/v3/transactions/{trans_id}/verify"
 
+    if token == "" or token is None:
+        raise TokenException(token=token, message="Authentication token absent")
+
     payload = {}
     headers = {
         'Authorization': f'Bearer {token}'
     }
 
     response = requests.request("GET", url, headers=headers, data=payload)
+    if response.status_code == 401:
+        raise TokenException(token=token, message='Invalid token provided')
+    if response.status_code == 400:
+        raise TransactionDetailException(trans_id=trans_id, message=f"{response.json()['message']}")
+    if response.status_code == 401:
+        raise TransactionDetailException(trans_id=trans_id, message=response.json()['message'])
     return dict(response.json())
 
 
-def trigger_mpesa_payment(tx_ref: str, amount: float, currency: str, email: Optional[str], phone_number: str,
-                          full_name: str) -> dict:
+def trigger_mpesa_payment(tx_ref: str, amount: float, currency: str, phone_number: str, full_name: str,
+                          email: Optional[str] = None) -> dict:
     """
     This will automatically trigger an MPESA payment in form of an STK Push from your customer. It will return a
     dictionary with details regarding the transaction. Flutterwave will also send the status to your webhook configured
     in the dashboard.
     """
     url = "https://api.flutterwave.com/v3/charges?type=mpesa"
-
+    if token == "" or token is None:
+        raise TokenException(token=token, message="Authentication token absent")
     payload = json.dumps({
         "tx_ref": f"{tx_ref}",
         "amount": f"{amount}",
@@ -76,6 +111,9 @@ def trigger_mpesa_payment(tx_ref: str, amount: float, currency: str, email: Opti
     }
 
     response = requests.request("POST", url, headers=headers, data=payload)
+    if response.status_code == 401:
+        raise TokenException(token=token, message='Invalid token provided')
+
     return dict(response.json())
 
 
@@ -117,6 +155,8 @@ def verify_bank_account_details(account_number: str, account_bank: str) -> dict:
     :param account_bank: str
     :return: dict
     """
+    if token == "" or token is None:
+        raise TokenException(token=token, message="Authentication token absent")
 
     url = "https://api.flutterwave.com/v3/accounts/resolve"
 
@@ -140,6 +180,8 @@ def verify_card_details(card_bin: str) -> dict:
     :return: dict
     """
     url = f"https://api.flutterwave.com/v3/card-bins/{card_bin}"
+    if token == "" or token is None:
+        raise TokenException(token=token, message="Authentication token absent")
 
     payload = {}
     headers = {
